# *******************************************************************************************
# 
# SharpCap script "Occultation_2023-06-19_menu_41.py"
# 2023/06/19 Jean-Francois
#
# The script works only with a PRO version of SharpCap and the ASCOM above 6.5.
#
# Version: 1.2.2:   Small bug correction
# Version: 1.2.1:   Save the weather conditions in the event file
# Version: 1.2.0:   Countdown and automatic capture start
# Version: 1.1.0:   First version
# Version: 1.0.0:   Test version
#
#
# The script is written for the following hardware:
# - QHY-174-GPS camera
# - ASCOM GOTO telescope
# - EAGLE computer with ECCO
# - A focuser is optional 
#
# Note: different hardware can be used, but all the automatic functions of the script are only with the above hardware.
# A different camera can be used, but without the GPS time and oder typical features of the camera.
# A telescope without GOTO can be used and an EAGLE computer is not necessary.
# The GPS information is read out from the GPS camera. Alternative GPS sensor maybe integrated in the future.
#
#
# The script has to be placed in the list of the Startup script list.
# In SharpCap, please follow these actions:

# - Copy the script somewhere on the computer
# - Start SharpCap
# - In "File" - "SharpCap Settings" - "Startup Scripts" - select this script
# - Close SharpCap (the script will be loaded at the next start)
# - Start SharpCap
# - Connect the QHY-174-GPS camera
# - Connect the telescope (and the focuser if available)
# - Read the script description document
#
# The LED Calibration script switch-on the LED automatically during the calibration process, if not activated.
# The LED Calibration script switch-off the LED automatically, if it was not activated before the start of the script.
#
# *******************************************************************************************


import datetime
import math
import os
import time
import urllib.request    # Web module
import clr
clr.AddReference("ASCOM.Tools")
clr.AddReference("System.Drawing")
clr.AddReference("System.IO")
clr.AddReference("System.Speech")
clr.AddReference("System.Windows.Forms")

import System.Drawing
import System.Speech.Synthesis as speech

from ASCOM.Tools import *
from datetime import datetime, timedelta, date
from math import *
from SharpCap.Base import CoordinateParser, RADecPosition, Epoch
from System import EventHandler
from System.Drawing import *
from System.IO import File
from System.Windows.Forms import *

# *******************************************************************************************
default_path = os.path.dirname(__file__)
LED_Calibration_script = default_path + "\Calibration_LED_2023-04-27_41.py"
default_icon           = default_path + "\Occultation.ico"


global DelayTime , EventDate, EventTime, EventDateTime, EAGLE

DelayTime = 0
Date_now = datetime.utcnow()
EventDate = Date_now.strftime("%d %B %Y")
EventTime = Date_now.strftime("%H:%M:%S")
EventDateTime = Date_now.strftime("%d %B %Y %H:%M:%S")

#EAGLE = "not known"     # If you have an EAGLE computer, uncomment this line and comment/delete the next line.
EAGLE = False

synth = speech.SpeechSynthesizer()
synth.SetOutputToDefaultAudioDevice()

print("Installed voices :")
for voice in synth.GetInstalledVoices():
    info = voice.VoiceInfo
    print(" Name: " + info.Name + ", " + str(info.Culture) + ", " + str(info.Gender))

langue = synth.Voice.Culture.TwoLetterISOLanguageName
print()
print("Selected Culture :", synth.Voice.Culture.EnglishName)
print("Selected Voice :",synth.Voice.Name)
print("Name :", langue)

# Choice of the voice
#UserCulture = System.Globalization.CultureInfo("fr-FR")
#synth.SelectVoiceByHints(VoiceGender.Male, VoiceAge.Adult, 0, UserCulture)

# Manual choice of the language from the list (en, de, fr, it)
#langue = "fr"
#print("Selected language :", langue)


def Speak(text):
    if (text == "Start capture"):
        if (langue == "en"):
            text = "start the capture"
        elif (langue == "de"):
            text = "start die Aufnahme"
        elif (langue == "fr"):
            text = "commencer la capture"
        elif (langue == "it"):
            text = "inizia a registrare"
   
    elif (text == "Stop capture"):
        if (langue == "en"):
            text = "stop the capture"
        elif (langue == "de"):
            text = "stop die Aufnahme"
        elif (langue == "fr"):
            text = "arrêter la capture"
        elif (langue == "it"):
            text = "interrompere la registrazione"

    elif (text == "Start countdown"):
        if (langue == "en"):
            text = "start countdown"
        elif (langue == "de"):
            text = "countdown starten"
        elif (langue == "fr"):
            text = "démarrer le compte à rebours"
        elif (langue == "it"):
            text = "inizia il conto alla rovescia"

    elif (text == "Stop countdown"):
        if (langue == "en"):
            text = "stop countdown"
        elif (langue == "de"):
            text = "countdown stoppen"
        elif (langue == "fr"):
            text = "arrêter le compte à rebours"
        elif (langue == "it"):
            text = "ferma il conto alla rovescia"

    elif (text == "one minute"):
        if (langue == "en"):
            text = "one minute"
        elif (langue == "de"):
            text = "eine Minute"
        elif (langue == "fr"):
            text = "une minute"
        elif (langue == "it"):
            text = "un minuto"

    elif (text == "10 seconds"):
        if (langue == "en"):
            text = "one minute"
        elif (langue == "de"):
            text = "10 Sekunden"
        elif (langue == "fr"):
            text = "10 secondes"
        elif (langue == "it"):
            text = "10 secondi"

    elif (text == "seconds"):
        if (langue == "en"):
            text = "seconds"
        elif (langue == "de"):
            text = "Sekunden"
        elif (langue == "fr"):
            text = "secondes"
        elif (langue == "it"):
            text = "secondi"

    synth.SpeakAsync(text)

def AfterCapture(file_in):
    lines = File.ReadAllLines(file_in)
    destination = os.path.dirname(SharpCap.GetLastCaptureFilename()) + "\\" + os.path.basename(file_in)
    File.WriteAllLines(destination, lines)          # Copy the report in the same directory as the SER film
    print() 
    print("Import a file: ", file_in)
    print("Saved file :", destination)
    print()
    try:
        with open(file_in, 'r') as fp:
            for l_no, line in enumerate(fp):
                print(line.replace("Â", "").replace("\n", ""))
        fp.close()
    except:
        print("File error")

def Roundtime(t):
    h, m, s = str(t).split(' ')[-1].split(":")
    s = round(float(s))
    if (s == 60):
        s = 0
        m = int(m) + 1
        if (m == 60):
            m = 0
            h = int(h) + 1
            if (h == 24):
                h = 0
    return(str(h).PadLeft(2,'0') + ":" + str(m).PadLeft(2,'0') + ":" + str(s).PadLeft(2,'0'))


class OccultationMenuForm(Form):
    def __init__(self):
        self.SuspendLayout()
        self.InitializeComponent()
        self.InitializeConnection()
        self.AutoScaleMode = System.Windows.Forms.AutoScaleMode.Dpi
        self.AutoScaleDimensions = SizeF(96, 96)
        self.ResumeLayout()
        self.tabControl.SelectedIndexChanged += EventHandler(self.TabControl_SelectedIndexChanged)

        print("Starting...")
        self.MyTimerUTC = Timer()
        self.MyTimerUTC.Interval = 1000
        self.MyTimerUTC.Tick += self.ShowUTC_Time

        try:
            delta = SharpCap.SelectedCamera.LatestStatus.AdditionalStatus["GPS"].NowShutterMicroSeconds/1000000
        except:
            delta = datetime.utcnow().microsecond/1000000
        time.sleep(delta)
        self.MyTimerUTC.Start()

    def InitializeComponent(self):
        self.Text = "Occultation"
        self.ClientSize = Size(550, 205)
        self.TopMost = True

        self.tabControl = TabControl()
        self.tabControl.Dock = DockStyle.Fill

        self.tabPage1 = TabPage()
        self.labelImportEvent = Label()
        self.textBoxImportFile = TextBox()
        self.buttonImport = Button()
        self.labelDate = Label()
        self.labelTime = Label()
        self.labelMaxDuration = Label()
        self.labelAst = Label()
        self.labelAstMag = Label()
        self.labelErrorInTime = Label()
        self.labelStar = Label()
        self.labelStarMag = Label()
        self.labelRA = Label()
        self.labelDEC = Label()
        self.labelTelStatus = Label()
        self.labelFocuser = Label()
        self.labelGPSCamera = Label()
        self.labelEAGLE = Label()
        self.labelGPS = Label()

        self.tabPage2 = TabPage()
        self.buttonGOTO = Button()
        self.buttonPlateSolve = Button()
        self.buttonLEDCal = Button()
        self.labelExposure = Label()
        self.textBoxExposure = TextBox()

        self.tabPage3 = TabPage()
        self.labelWind = Label()
        self.comboboxWind = ComboBox()
        self.labelTemperature = Label()
        self.textBoxTemperature = TextBox()
        self.labelTransparency = Label()
        self.comboboxTransparency = ComboBox()
        self.labelStability = Label()
        self.comboboxStability = ComboBox()
        self.labelSODISComment = Label()
        self.textSODISComment = TextBox()
        self.labelNote = Label()
        self.textNote = TextBox()
        self.buttonWeatherSave = Button()

        self.tabPage4 = TabPage()
        self.checkboxAutoCapture = CheckBox()
        self.buttonStart = Button()
        self.labelDelay = Label()
        self.textBoxTimeDelay = TextBox()
        self.labelUTC = Label()
        self.labelCountdown = Label()
        self.textUTCTime = TextBox()
        self.textCountdown = TextBox()

        self.Controls.Add(self.tabControl)
        self.tabControl.TabPages.Add(self.tabPage1)
        self.tabControl.TabPages.Add(self.tabPage2)
        self.tabControl.TabPages.Add(self.tabPage3)
        self.tabControl.TabPages.Add(self.tabPage4)

        self.tabControl.SelectedIndex = 0;
        self.tabControl.TabIndex = 1;
        self.tabControl.Padding = Point(20, 2)
        self.tabControl.ShowToolTips = True
        self.tabPage1.ToolTipText = "Import event file and show equipment status."
        self.tabPage2.ToolTipText = "Perform a GOTO and Plate Solve only if a telescope is connected.\nStart the LED calibration only with QHY-174GPS camera"
        self.tabPage3.ToolTipText = "Set weather conditions and additional note.\nUse SODIS Comment only for important comment."
        self.tabPage4.ToolTipText = "Set the start delay before the predicted time event.\nStart the countdown for the automatic capture."

        #tabPage1
        self.tabPage1.Controls.Add(self.labelImportEvent)
        self.tabPage1.Controls.Add(self.textBoxImportFile)
        self.tabPage1.Controls.Add(self.buttonImport)
        self.tabPage1.Controls.Add(self.labelDate)
        self.tabPage1.Controls.Add(self.labelTime)
        self.tabPage1.Controls.Add(self.labelMaxDuration)
        self.tabPage1.Controls.Add(self.labelAst)
        self.tabPage1.Controls.Add(self.labelAstMag)
        self.tabPage1.Controls.Add(self.labelErrorInTime)
        self.tabPage1.Controls.Add(self.labelStar)
        self.tabPage1.Controls.Add(self.labelStarMag)
        self.tabPage1.Controls.Add(self.labelRA)
        self.tabPage1.Controls.Add(self.labelDEC)
        self.tabPage1.Controls.Add(self.labelTelStatus)
        self.tabPage1.Controls.Add(self.labelFocuser)
        self.tabPage1.Controls.Add(self.labelGPSCamera)
        self.tabPage1.Controls.Add(self.labelEAGLE)
        self.tabPage1.Controls.Add(self.labelGPS)
        self.tabPage1.Name = "tabPage1"
        self.tabPage1.TabIndex = 0
        self.tabPage1.Text = "Event Info"
        self.tabPage1.UseVisualStyleBackColor = True

        self.labelImportEvent.AutoSize = True
        self.labelImportEvent.Location = Point(10, 10)
        self.labelImportEvent.Name = "labelImportEvent"
        self.labelImportEvent.Size = Size(70, 13)
        self.labelImportEvent.Text = "Import Event:"

        self.textBoxImportFile.AutoSize = True
        self.textBoxImportFile.Location = Point(95, 7)
        self.textBoxImportFile.Name = "textBoxImportFile"
        self.textBoxImportFile.Size = Size(340, 20)
        self.textBoxImportFile.Text = "Import a file"

        self.buttonImport.Font = Font("Microsoft Sans Serif", 10, FontStyle.Bold)
        self.buttonImport.Location = Point(455, 5)
        self.buttonImport.Name = "buttonImport"
        self.buttonImport.Text = "Import"
        self.buttonImport.AutoSize = True
        self.buttonImport.Click += self.ImportEvent
        self.buttonImport.UseVisualStyleBackColor = True

        self.labelDate.AutoSize = True
        self.labelDate.Location = Point(10, 40)
        self.labelDate.Name = "labelDate"
        self.labelDate.Size = Size(33, 13)
        self.labelDate.Text = "Date:"

        self.labelTime.AutoSize = True
        self.labelTime.Location = Point(230, 40)
        self.labelTime.Name = "labelTime"
        self.labelTime.Size = Size(33, 13)
        self.labelTime.Text = "Time:"

        self.labelMaxDuration.AutoSize = True
        self.labelMaxDuration.Location = Point(400, 40)
        self.labelMaxDuration.Name = "labelMaxDuration"
        self.labelMaxDuration.Size = Size(33, 13)
        self.labelMaxDuration.Text = "Max. Duration:"

        self.labelAst.AutoSize = True
        self.labelAst.Location = Point(10, 60)
        self.labelAst.Name = "labelAst"
        self.labelAst.Size = Size(48, 13)
        self.labelAst.Text = "Asteroid:"

        self.labelAstMag.AutoSize = True
        self.labelAstMag.Location = Point(230, 60)
        self.labelAstMag.Name = "labelAstMag"
        self.labelAstMag.Size = Size(31, 13)
        self.labelAstMag.Text = "Ast Mag:"

        self.labelErrorInTime.AutoSize = True
        self.labelErrorInTime.Location = Point(400, 60)
        self.labelErrorInTime.Name = "labelErrorInTime"
        self.labelErrorInTime.Size = Size(31, 13)
        self.labelErrorInTime.Text = "Error in Time:"

        self.labelStar.AutoSize = True
        self.labelStar.Location = Point(10, 80)
        self.labelStar.Name = "labelStar"
        self.labelStar.Size = Size(29, 1)
        self.labelStar.Text = "Star:"

        self.labelStarMag.AutoSize = True
        self.labelStarMag.Location = Point(230, 80)
        self.labelStarMag.Name = "labelStarMag"
        self.labelStarMag.Size = Size(31, 13)
        self.labelStarMag.Text = "Star Mag:"

        self.labelRA.AutoSize = True
        self.labelRA.Location = Point(10, 100)
        self.labelRA.Name = "labelR"
        self.labelRA.Size = Size(25, 1)
        self.labelRA.Text = "RA:"

        self.labelDEC.AutoSize = True
        self.labelDEC.Location = Point(230, 100)
        self.labelDEC.Name = "labelDE"
        self.labelDEC.Size = Size(32, 13)
        self.labelDEC.Text = "DEC:"

        self.labelTelStatus.AutoSize = True
        self.labelTelStatus.Location = Point(10, 120)
        self.labelTelStatus.Name = "labelTelStatus"
        self.labelTelStatus.Size = Size(123, 13)

        self.labelFocuser.AutoSize = True
        self.labelFocuser.Location = Point(230, 120)
        self.labelFocuser.Name = "labelFocuser"
        self.labelFocuser.Size = Size(111, 13)

        self.labelGPSCamera.AutoSize = True
        self.labelGPSCamera.Location = Point(10, 140)
        self.labelGPSCamera.Name = "labelGPSCamera"
        self.labelGPSCamera.Size = Size(121, 13)

        self.labelGPS.AutoSize = True
        self.labelGPS.Location = Point(230, 140)
        self.labelGPS.Name = "labelGPS"
        self.labelGPS.Size = Size(121, 13)

        self.labelEAGLE.AutoSize = True
        self.labelEAGLE.Location = Point(10, 160)
        self.labelEAGLE.Name = "labelEAGLE"
        self.labelEAGLE.Size = Size(108, 13)

        #tabPage2
        self.tabPage2.Controls.Add(self.buttonGOTO)
        self.tabPage2.Controls.Add(self.buttonPlateSolve)
        self.tabPage2.Controls.Add(self.buttonLEDCal)
        self.tabPage2.Controls.Add(self.labelExposure)
        self.tabPage2.Controls.Add(self.textBoxExposure)

        self.tabPage2.Name = "tabPage2"
        self.tabPage2.TabIndex = 1
        self.tabPage2.Text = "Telescope/Camera"
        self.tabPage2.UseVisualStyleBackColor = True

        self.buttonGOTO.Font = Font("Microsoft Sans Serif", 13, FontStyle.Bold)
        self.buttonGOTO.Location = Point(8, 10)
        self.buttonGOTO.Name = "buttonGOTO"
        self.buttonGOTO.Size = Size(170, 60)
        self.buttonGOTO.Text = "GOTO"
        self.buttonGOTO.Click += self.GOTO
        self.buttonGOTO.UseVisualStyleBackColor = True

        self.buttonPlateSolve.Font = Font("Microsoft Sans Serif", 13, FontStyle.Bold)
        self.buttonPlateSolve.Location = Point(190, 10)
        self.buttonPlateSolve.Name = "buttonPlateSolve"
        self.buttonPlateSolve.Size = Size(170, 60)
        self.buttonPlateSolve.Text = "Plate Solve"
        self.buttonPlateSolve.Click += self.PlateSolve
        self.buttonPlateSolve.UseVisualStyleBackColor = True

        self.buttonLEDCal.Font = Font("Microsoft Sans Serif", 13, FontStyle.Bold)
        self.buttonLEDCal.Location = Point(370, 10)
        self.buttonLEDCal.Name = "buttonLEDCal"
        self.buttonLEDCal.Size = Size(170, 60)
        self.buttonLEDCal.Text = "LED Calibration"
        self.buttonLEDCal.Click += self.LEDCalibration
        self.buttonLEDCal.UseVisualStyleBackColor = True

        self.labelExposure.AutoSize = True
        self.labelExposure.Location = Point(195, 80)
        self.labelExposure.Name = "labelExposure"
        self.labelExposure.Size = Size(70, 13)
        self.labelExposure.Text = "Camera Exposure [s]:"

        self.textBoxExposure.AutoSize = True
        self.textBoxExposure.Location = Point(320, 78)
        self.textBoxExposure.Name = "textBoxExposure"
        self.textBoxExposure.Size = Size(30, 20)
        self.textBoxExposure.Text = "0"

        #tabPage3
        self.tabPage3.Controls.Add(self.labelWind)
        self.tabPage3.Controls.Add(self.comboboxWind)
        self.tabPage3.Controls.Add(self.labelTemperature)
        self.tabPage3.Controls.Add(self.textBoxTemperature)
        self.tabPage3.Controls.Add(self.labelTransparency)
        self.tabPage3.Controls.Add(self.comboboxTransparency)
        self.tabPage3.Controls.Add(self.labelStability)
        self.tabPage3.Controls.Add(self.comboboxStability)
        self.tabPage3.Controls.Add(self.labelSODISComment)
        self.tabPage3.Controls.Add(self.textSODISComment)
        self.tabPage3.Controls.Add(self.labelNote)
        self.tabPage3.Controls.Add(self.textNote)
        self.tabPage3.Controls.Add(self.buttonWeatherSave)

        self.tabPage3.Name = "tabPage3"
        self.tabPage3.TabIndex = 2
        self.tabPage3.Text = "Weather/Info"
        self.tabPage3.UseVisualStyleBackColor = True

        self.labelWind.AutoSize = True
        self.labelWind.Location = Point(10, 10)
        self.labelWind.Name = "labelWind"
        self.labelWind.Text = "Wind:"

        self.comboboxWind.Location = Point(50, 7)
        self.comboboxWind.Size = Size(130,10)
        self.comboboxWind.Items.Add("unstated")
        self.comboboxWind.Items.Add("0: Calm")
        self.comboboxWind.Items.Add("1: Light air")
        self.comboboxWind.Items.Add("2: Light breeze")
        self.comboboxWind.Items.Add("3: Gentle breeze")
        self.comboboxWind.Items.Add("4: Moderate breeze")
        self.comboboxWind.Items.Add("5: Fresh breeze")
        self.comboboxWind.Items.Add("6: Strong breeze")
        self.comboboxWind.Items.Add("7: High wind")
        self.comboboxWind.Items.Add("8: Gale")
        self.comboboxWind.Items.Add("9: Strong gale")
        self.comboboxWind.SelectedItem  = "unstated"

        self.labelTemperature.AutoSize = True
        self.labelTemperature.Location = Point(10, 35)
        self.labelTemperature.Name = "labelTemperature"
        self.labelTemperature.Text = "Temperature:"

        self.textBoxTemperature.AutoSize = True
        self.textBoxTemperature.Location = Point(90, 32)
        self.textBoxTemperature.Name = "textBoxTemperature"
        self.textBoxTemperature.Size = Size(40, 20)
        self.textBoxTemperature.Text = ""

        self.labelTransparency.AutoSize = True
        self.labelTransparency.Location = Point(200, 10)
        self.labelTransparency.Name = "labelTransparency"
        self.labelTransparency.Text = "Transparency:"

        self.comboboxTransparency.Location = Point(280, 7)
        self.comboboxTransparency.Size = Size(150,10)
        self.comboboxTransparency.Items.Add("unstated")
        self.comboboxTransparency.Items.Add("Clear")
        self.comboboxTransparency.Items.Add("Fog")
        self.comboboxTransparency.Items.Add("Thin cloud")
        self.comboboxTransparency.Items.Add("Thick cloud")
        self.comboboxTransparency.Items.Add("Broken opaque cloud")
        self.comboboxTransparency.Items.Add("Star faint")
        self.comboboxTransparency.Items.Add("By averted vision")
        self.comboboxTransparency.SelectedItem  = "unstated"

        self.labelStability.AutoSize = True
        self.labelStability.Location = Point(200, 35)
        self.labelStability.Name = "labelStability"
        self.labelStability.Text = "Stability:"

        self.comboboxStability.Location = Point(280, 32)
        self.comboboxStability.Size = Size(150,10)
        self.comboboxStability.Items.Add("unstated")
        self.comboboxStability.Items.Add("Steady")
        self.comboboxStability.Items.Add("Slight flickering")
        self.comboboxStability.Items.Add("Strong flickering")
        self.comboboxStability.SelectedItem  = "unstated"

        self.labelSODISComment.AutoSize = True
        self.labelSODISComment.Location = Point(10, 65)
        self.labelSODISComment.Name = "labelSODISComment"
        self.labelSODISComment.Text = "SODIS Comment:"

        self.textSODISComment.AutoSize = True
        self.textSODISComment.Location = Point(130, 65)
        self.textSODISComment.Name = "textSODISComment"
        self.textSODISComment.Size = Size(370, 50)
        self.textSODISComment.Text = ""
        self.textSODISComment.Multiline = True

        self.labelNote.AutoSize = True
        self.labelNote.Location = Point(10, 120)
        self.labelNote.Name = "labelNote"
        self.labelNote.Text = "Note:"

        self.textNote.AutoSize = True
        self.textNote.Location = Point(130, 120)
        self.textNote.Name = "textNote"
        self.textNote.Size = Size(370, 50)
        self.textNote.Text = ""
        self.textNote.Multiline = True

        self.buttonWeatherSave.Font = Font("Microsoft Sans Serif", 10, FontStyle.Bold)
        self.buttonWeatherSave.Location = Point(440, 7)
        self.buttonWeatherSave.Name = "buttonWeatherSave"
        self.buttonWeatherSave.Size = Size(100, 50)
        self.buttonWeatherSave.Text = "Save information"
        self.buttonWeatherSave.Click += self.WeatherSave
        self.buttonWeatherSave.UseVisualStyleBackColor = True

        #tabPage4
        self.tabPage4.Controls.Add(self.checkboxAutoCapture)
        self.tabPage4.Controls.Add(self.buttonStart)
        self.tabPage4.Controls.Add(self.labelDelay)
        self.tabPage4.Controls.Add(self.textBoxTimeDelay)
        self.tabPage4.Controls.Add(self.labelUTC)
        self.tabPage4.Controls.Add(self.labelCountdown)
        self.tabPage4.Controls.Add(self.textUTCTime)
        self.tabPage4.Controls.Add(self.textCountdown)

        self.tabPage4.Name = "tabPage4"
        self.tabPage4.TabIndex = 3
        self.tabPage4.Text = "Capture"
        self.tabPage4.UseVisualStyleBackColor = True

        self.checkboxAutoCapture.Text = "Automatic Capture"
        self.checkboxAutoCapture.Location = Point(15, 17)
        self.checkboxAutoCapture.AutoSize = True
        self.checkboxAutoCapture.CheckedChanged += self.CheckBoxChanged

        self.buttonStart.Font = Font("Microsoft Sans Serif", 15, FontStyle.Bold)
        self.buttonStart.Location = Point(150, 9)
        self.buttonStart.Name = "buttonStart"
        self.buttonStart.Size = Size(220, 31)
        self.buttonStart.Text = "Start Capture"
        self.buttonStart.UseVisualStyleBackColor = True
        self.buttonStart.Click += self.StartStop

        self.labelDelay.AutoSize = True
        self.labelDelay.Location = Point(390, 17)
        self.labelDelay.Name = "labelDelay"
        self.labelDelay.Text = "Delay [s]:"

        self.textBoxTimeDelay.AutoSize = True
        self.textBoxTimeDelay.Location = Point(460, 16)
        self.textBoxTimeDelay.Name = "textBoxTimeDelay"
        self.textBoxTimeDelay.Size = Size(60, 20)
        self.textBoxTimeDelay.Text = "0"

        self.labelUTC.AutoSize = True
        self.labelUTC.Location = Point(7, 54)
        self.labelUTC.Name = "labelUTC"
        self.labelUTC.Text = "UTC-GPS / Computer:"

        self.labelCountdown.AutoSize = True
        self.labelCountdown.Location = Point(280, 54)
        self.labelCountdown.Name = "labelCountdown"
        self.labelCountdown.Text = "Countdown:"

        self.textUTCTime.Font = Font("Microsoft Sans Serif", 45, FontStyle.Bold)
        self.textUTCTime.Location = Point(7, 70)
        self.textUTCTime.Name = "textUTCTime"
        self.textUTCTime.Size = Size(255, 45)
        self.textUTCTime.Text = "UTC Time"

        self.textCountdown.Font = Font("Microsoft Sans Serif", 45, FontStyle.Bold)
        self.textCountdown.Location = Point(280, 70)
        self.textCountdown.Name = "textCountdown"
        self.textCountdown.Size = Size(255, 45)
        self.textCountdown.Text = "Countdown"

    def InitializeConnection(self):
        global Star_RA, Star_DEC, TelCanGoto, EventTime, EAGLE

        # Telescope connection
        if (SharpCap.Mounts.SelectedMount == None):
            self.labelTelStatus.Text = "Telescope: not selected"
        else:
            if (SharpCap.Mounts.SelectedMount.Connected):
                self.labelTelStatus.Text = "Telescope: " + str(SharpCap.Mounts.SelectedMount)
                if (SharpCap.Mounts.SelectedMount.CanGoto):
                    TelCanGoto = True
                    self.labelTelStatus.ForeColor = Color.Green
                    self.buttonGOTO.BackColor = Color.Green
                    self.buttonGOTO.ForeColor = Color.White
                    self.buttonGOTO.Enabled = True
                    self.buttonPlateSolve.Enabled = True
                    self.buttonPlateSolve.BackColor = Color.Green
                    self.buttonPlateSolve.ForeColor = Color.White
                else:
                    TelCanGoto = False
                    self.buttonGOTO.BackColor = Color.Maroon
                    self.buttonGOTO.ForeColor = Color.White
                    self.buttonGOTO.Enabled = False
                    self.buttonPlateSolve.Enabled = False
                    self.buttonPlateSolve.BackColor = Color.Maroon
                    self.buttonPlateSolve.ForeColor = Color.White
            else:
                if (SharpCap.NightModeControl.Enabled):
                    self.labelTelStatus.ForeColor = Color.DarkOrange
                else:
                    self.labelTelStatus.ForeColor = Color.Black
                self.buttonGOTO.BackColor = Color.Maroon
                self.buttonGOTO.ForeColor = Color.White
                self.buttonGOTO.Enabled = False
                self.buttonPlateSolve.BackColor = Color.Maroon
                self.buttonPlateSolve.ForeColor = Color.White
                self.buttonPlateSolve.Enabled = False
                self.labelTelStatus.Text = "Telescope: not connected"

        # Focuser connection
        if (SharpCap.Focusers.SelectedFocuser == None):
            self.labelFocuser.Text = "Focuser: not selected"
        else:
            if (SharpCap.Focusers.SelectedFocuser.Connected):
                Focuser_Temp = ""
                try:
                    Focuser_Temp = SharpCap.Focusers.SelectedFocuser.Temperature
                except:
                    print("No temperature sensor on the focuser")
                self.labelFocuser.Text = "Focuser: " + str(SharpCap.Focusers.SelectedFocuser) + ", " + str("%.1f" % round(Focuser_Temp,1)) + "°C"
                self.labelFocuser.ForeColor = Color.Green
                self.textBoxTemperature.Text = str("%.1f" % round(Focuser_Temp,1))
            else:
                if (SharpCap.NightModeControl.Enabled):
                    self.labelFocuser.ForeColor = Color.DarkOrange
                else:
                    self.labelFocuser.ForeColor = Color.Black
                self.labelFocuser.Text = "Focuser: not connected"

        # Camera connection
        if (SharpCap.SelectedCamera == None):
            self.labelGPSCamera.Text = "Camera: not selected"
            if (SharpCap.NightModeControl.Enabled):
                self.labelGPSCamera.ForeColor = Color.DarkOrange
            else:
                self.labelGPSCamera.ForeColor = Color.Black
        else:
            try:
                GPS = str(SharpCap.SelectedCamera.Controls.FindByName("GPS").Value)
                if (GPS == "On" or GPS == "Off"):
                    GPS = True
                    self.buttonLEDCal.BackColor = Color.Green
                    self.buttonLEDCal.ForeColor = Color.White
                    print("GPS camera detected")
            except:
                GPS = False
                print("No GPS camera detected")
                self.buttonLEDCal.Enabled = False
                self.buttonLEDCal.BackColor = Color.Maroon
            string_camera = str(SharpCap.SelectedCamera)
            camera_name = string_camera.split("::")[1]
            if (GPS):
                self.labelGPSCamera.Text = "Camera: " + camera_name + "-GPS"
                self.labelGPSCamera.ForeColor = Color.Green
            else:
                self.labelGPSCamera.Text = "Camera: " + camera_name
                self.labelGPSCamera.ForeColor = Color.Green

        # GPS connection (from QHY174-GPS)
        if (SharpCap.SelectedCamera == None):
            self.labelGPS.Text = "GPS: no camera detected"
            if (SharpCap.NightModeControl.Enabled):
                self.labelGPS.ForeColor = Color.DarkOrange
            else:
                self.labelGPS.ForeColor = Color.Black
        else:
            try:
                GPS = str(SharpCap.SelectedCamera.Controls.FindByName("GPS").Value)
                if (GPS == "On" or GPS == "Off"):
                    GPS = True
                    print("GPS detected")
            except:
                GPS = False
                print("No GPS detected")

            string_camera = str(SharpCap.SelectedCamera)
            camera_name = string_camera.split("::")[1]
            if (GPS):
                try:
                    GPS_status = str(SharpCap.SelectedCamera.LatestStatus.AdditionalStatus["GPS"].Status)
                    GPS_status = GPS_status + ", " + str(SharpCap.SelectedCamera.LatestStatus.AdditionalStatus["GPS"].FixQuality)
                    GPS_status = GPS_status + ", " + str(SharpCap.SelectedCamera.LatestStatus.AdditionalStatus["GPS"].NumberOfSatellites) + " sats"
                    GPS_status = GPS_status + ", Lat: " + str("%.4f" % round(SharpCap.SelectedCamera.LatestStatus.AdditionalStatus["GPS"].Latitude,6)) + "°"
                    GPS_status = GPS_status + ", Lon: " + str("%.4f" % round(SharpCap.SelectedCamera.LatestStatus.AdditionalStatus["GPS"].Longitude,6)) + "°"
                    self.labelGPS.Text = "GPS: " + GPS_status
                    self.labelGPS.ForeColor = Color.Green
                except:
                    self.labelGPS.ForeColor = Color.Red
                    self.labelGPS.Text = "GPS: GPS not activated"
            else:
                self.labelGPS.Text = "GPS: No GPS detected"

        # EAGLE connection (PrimaLuceLab)
        self.labelEAGLE.Text = "EAGLE: not detected"
        if (EAGLE == "not known" or EAGLE == True):
            try:
                eagle_all = urllib.request.urlopen("http://localhost:1380/getecco")
                eagle_byte = eagle_all.read()
                eagle_string = eagle_byte.ToString()
                eagle = eagle_string.replace('"','')
                eagle = eagle.replace(":",": ")
                eagle = eagle.replace("b\'{","")
                eagle = eagle.replace("}\'","")
                eagle = eagle.replace("hum","humidity")
                b = eagle.Split(",")
                eagle_info = b[2] + "°C, " + b[3] + "%, " + b[4] + " °C, " + b[5] + " mBar"
                #eagle_info = " EAGLE info, temperature, humidity, pressure"
                EAGLE = True
                print("EAGLE info: " + eagle_info)
                self.labelEAGLE.Text = "EAGLE: connected, " + eagle_info
            except:
                EAGLE = False
                self.labelEAGLE.Text = "EAGLE: not detected"
                print("EAGLE: not detected")

        if (SharpCap.NightModeControl.Enabled):
            self.StyleFromParent = False
            self.BackColor = Color.Black

            self.tabPage1.BackColor = Color.Black
            self.labelImportEvent.BackColor = Color.Black
            self.textBoxImportFile.BackColor = Color.Black
            self.buttonImport.BackColor = Color.Black

            self.labelImportEvent.ForeColor = Color.DarkOrange
            self.textBoxImportFile.ForeColor = Color.DarkOrange
            self.buttonImport.ForeColor = Color.DarkOrange
            self.labelDate.ForeColor = Color.DarkOrange
            self.labelTime.ForeColor = Color.DarkOrange
            self.labelMaxDuration.ForeColor = Color.DarkOrange
            self.labelAst.ForeColor = Color.DarkOrange
            self.labelAstMag.ForeColor = Color.DarkOrange
            self.labelErrorInTime.ForeColor = Color.DarkOrange
            self.labelStar.ForeColor = Color.DarkOrange
            self.labelStarMag.ForeColor = Color.DarkOrange
            self.labelRA.ForeColor = Color.DarkOrange
            self.labelDEC.ForeColor = Color.DarkOrange
            self.labelTelStatus.ForeColor = Color.DarkOrange
            self.labelFocuser.ForeColor = Color.DarkOrange
            self.labelGPSCamera.ForeColor = Color.DarkOrange
            self.labelGPS.ForeColor = Color.DarkOrange
            self.labelEAGLE.ForeColor = Color.DarkOrange
            self.labelGPS.ForeColor = Color.DarkOrange

            self.tabPage2.BackColor = Color.Black
            self.labelExposure.BackColor = Color.Black
            self.textBoxExposure.BackColor = Color.Black
            self.labelExposure.ForeColor = Color.DarkOrange
            self.textBoxExposure.ForeColor = Color.DarkOrange

            self.tabPage3.BackColor = Color.Black
            self.labelWind.BackColor = Color.Black
            self.comboboxWind.BackColor = Color.Black
            self.labelTemperature.BackColor = Color.Black
            self.textBoxTemperature.BackColor = Color.Black
            self.labelTransparency.BackColor = Color.Black
            self.comboboxTransparency.BackColor = Color.Black
            self.labelStability.BackColor = Color.Black
            self.comboboxStability.BackColor = Color.Black
            self.labelSODISComment.BackColor = Color.Black
            self.textSODISComment.BackColor = Color.Black
            self.labelNote.BackColor = Color.Black
            self.textNote.BackColor = Color.Black

            self.labelWind.ForeColor = Color.DarkOrange
            self.comboboxWind.ForeColor = Color.DarkOrange
            self.labelTemperature.ForeColor = Color.DarkOrange
            self.textBoxTemperature.ForeColor = Color.DarkOrange
            self.labelTransparency.ForeColor = Color.DarkOrange
            self.comboboxTransparency.ForeColor = Color.DarkOrange
            self.labelStability.ForeColor = Color.DarkOrange
            self.comboboxStability.ForeColor = Color.DarkOrange
            self.labelSODISComment.ForeColor = Color.DarkOrange
            self.textSODISComment.ForeColor = Color.DarkOrange
            self.labelNote.ForeColor = Color.DarkOrange
            self.textNote.ForeColor = Color.DarkOrange

            self.tabPage4.BackColor = Color.Black
            self.checkboxAutoCapture.BackColor = Color.Gray
            self.buttonStart.BackColor = Color.Black
            self.labelDelay.BackColor = Color.Black
            self.textBoxTimeDelay.BackColor = Color.Black
            self.labelUTC.BackColor = Color.Black
            self.labelCountdown.BackColor = Color.Black
            self.textUTCTime.BackColor = Color.Black
            self.textCountdown.BackColor = Color.Black

            self.checkboxAutoCapture.ForeColor = Color.DarkOrange
            self.buttonStart.ForeColor = Color.DarkOrange
            self.labelDelay.ForeColor = Color.DarkOrange
            self.textBoxTimeDelay.ForeColor = Color.DarkOrange
            self.labelUTC.ForeColor = Color.DarkOrange
            self.labelCountdown.ForeColor = Color.DarkOrange
            self.textUTCTime.ForeColor = Color.DarkOrange
            self.textCountdown.ForeColor = Color.DarkOrange

        if (not SharpCap.NightModeControl.Enabled):
            self.StyleFromParent = True
            self.BackColor = Color.White
            self.tabPage1.BackColor = Color.White
            self.labelImportEvent.BackColor = Color.White
            self.textBoxImportFile.BackColor = Color.White
            self.buttonImport.BackColor = Color.White

            self.labelImportEvent.ForeColor = Color.Black
            self.textBoxImportFile.ForeColor = Color.Black
            self.buttonImport.ForeColor = Color.Black
            self.labelDate.ForeColor = Color.Black
            self.labelTime.ForeColor = Color.Black
            self.labelMaxDuration.ForeColor = Color.Black
            self.labelAst.ForeColor = Color.Black
            self.labelAstMag.ForeColor = Color.Black
            self.labelErrorInTime.ForeColor = Color.Black
            self.labelStar.ForeColor = Color.Black
            self.labelStarMag.ForeColor = Color.Black
            self.labelRA.ForeColor = Color.Black
            self.labelDEC.ForeColor = Color.Black

            self.tabPage2.BackColor = Color.White
            self.labelExposure.BackColor = Color.White
            self.textBoxExposure.BackColor = Color.White
            self.labelExposure.ForeColor = Color.Black
            self.textBoxExposure.ForeColor = Color.Black

            self.tabPage3.BackColor = Color.White
            self.labelWind.BackColor = Color.White
            self.comboboxWind.BackColor = Color.White
            self.labelTemperature.BackColor = Color.White
            self.textBoxTemperature.BackColor = Color.White
            self.labelTransparency.BackColor = Color.White
            self.comboboxTransparency.BackColor = Color.White
            self.labelStability.BackColor = Color.White
            self.comboboxStability.BackColor = Color.White
            self.labelSODISComment.BackColor = Color.White
            self.textSODISComment.BackColor = Color.White
            self.labelNote.BackColor = Color.White
            self.textNote.BackColor = Color.White

            self.labelWind.ForeColor = Color.Black
            self.comboboxWind.ForeColor = Color.Black
            self.labelTemperature.ForeColor = Color.Black
            self.textBoxTemperature.ForeColor = Color.Black
            self.labelTransparency.ForeColor = Color.Black
            self.comboboxTransparency.ForeColor = Color.Black
            self.labelStability.ForeColor = Color.Black
            self.comboboxStability.ForeColor = Color.Black
            self.labelSODISComment.ForeColor = Color.Black
            self.textSODISComment.ForeColor = Color.Black
            self.labelNote.ForeColor = Color.Black
            self.textNote.ForeColor = Color.Black

            self.tabPage4.BackColor = Color.White
            self.checkboxAutoCapture.BackColor = Color.White
            self.buttonStart.BackColor = Color.White
            self.labelDelay.BackColor = Color.White
            self.textBoxTimeDelay.BackColor = Color.White
            self.labelUTC.BackColor = Color.White
            self.labelCountdown.BackColor = Color.White
            self.textUTCTime.BackColor = Color.White
            self.textCountdown.BackColor = Color.White

            self.checkboxAutoCapture.ForeColor = Color.Black
            self.buttonStart.ForeColor = Color.Black
            self.labelDelay.ForeColor = Color.Black
            self.textBoxTimeDelay.ForeColor = Color.Black
            self.labelUTC.ForeColor = Color.Black
            self.labelCountdown.ForeColor = Color.Black
            self.textUTCTime.ForeColor = Color.Black
            self.textCountdown.ForeColor = Color.Black

    def CheckBoxChanged(self, sender, args):
        if (self.checkboxAutoCapture.Checked):
            if (self.buttonStart.Text == "Start Capture"):
                self.buttonStart.Text = "Start Countdown"
                self.buttonStart.BackColor = Color.Green
                self.buttonStart.ForeColor = Color.White
        else:
            self.buttonStart.Text = "Start Capture"
            if (SharpCap.NightModeControl.Enabled):
                self.buttonStart.BackColor = Color.Black
                self.buttonStart.ForeColor = Color.DarkOrange
            else:
                self.buttonStart.BackColor = Color.White
                self.buttonStart.ForeColor = Color.Black

    def StartStop(self, sender, args):
        if (self.buttonStart.Text == "Start Capture"):
            self.StartCapture()
            print("Start Capture")
            Speak("Start capture")
            temp_text = "Stop Capture"
            self.checkboxAutoCapture.Enabled = False

        if (self.buttonStart.Text == "Stop Capture"):
            SharpCap.SelectedCamera.StopCapture()
            self.WeatherSave(self, sender)
            AfterCapture(self.textBoxImportFile.Text)
            print("Stop Capture")
            temp_text = "Start Capture"
            Speak("Stop capture")
            self.checkboxAutoCapture.Enabled = True

        if (self.buttonStart.Text == "Start Countdown"):
            print("Start Countdown")
            temp_text = "Stop Countdown"
            Speak("Start countdown")
            self.buttonStart.BackColor = Color.Red
            self.checkboxAutoCapture.Enabled = False

        if (self.buttonStart.Text == "Stop Countdown"):
            print("Stop Countdown")
            temp_text = "Start Countdown"
            Speak("Stop countdown")
            self.buttonStart.BackColor = Color.Green
            self.checkboxAutoCapture.Enabled = True

        self.buttonStart.Text = temp_text

    def GOTO(self, sender, args):
        global Star_RA, Star_DEC
        if (self.buttonGOTO.Text == "GOTO"):
            self.buttonGOTO.Text = "STOP"
            self.buttonGOTO.BackColor = Color.Red
            self.buttonGOTO.ForeColor = Color.White
            coordinates = CoordinateParser.Parse(Star_RA + ";" + Star_DEC, True)
            print("radp: ", coordinates)
            print()
            print("Star RA : " + Star_RA.replace("\n", "") + ", " + str("%.6f" % round(coordinates.RightAscension,6)))
            #print("Star RA : " + str("%.6f" % round(coordinates.RightAscension,6)))
            print("Star DEC: " + Star_DEC.replace("\n", "") + ", " + str("%.6f" % round(coordinates.Declination,6)))
            #print("Star DEC: " + str("%.6f" % round(coordinates.Declination,6))) 
            print("Slew to target")
            SharpCap.Mounts.SelectedMount.SlewTo(coordinates)

            time.sleep(2)       #do not delete ! important for the STOP text

        else:
            SharpCap.Mounts.SelectedMount.Stop()
            print("Stop the GOTO")

    def PlateSolve(self, sender, args):
        global Star_RA, Star_DEC
        try:
            start_exposure = SharpCap.SelectedCamera.Controls.Exposure.Value
            new_exposure = float(self.textBoxExposure.Text)
            if new_exposure != 0.0:
                SharpCap.SelectedCamera.Controls.Exposure.Value = 1000 * new_exposure
            time.sleep(1.1 * new_exposure)
            SharpCap.Mounts.SelectedMount.SolveAndSync()
            coordinates = CoordinateParser.Parse(Star_RA + ";" + Star_DEC, True)
            SharpCap.Mounts.SelectedMount.SlewTo(coordinates)
            SharpCap.SelectedCamera.Controls.Exposure.Value = start_exposure
        except:
            print("Problem with Plate Solve or with GOTO")

    def ImportEvent(self, sender, args):
        global Star_RA, Star_DEC, EventDate, EventTime, EventDateTime
        filename = OpenFileDialog()
        filename.Title = "Open SODIS template";
        filename.DefaultExt = "txt";
        filename.Filter = "Text files (*.txt)|*.txt|All files (*.*)|*.*";
        filename.Multiselect = False;

        if (filename.ShowDialog() == DialogResult.OK):
            self.textBoxImportFile.Text = filename.FileName
            print()
            with open(filename.FileName, 'r') as fp:
                for l_no, line in enumerate(fp):
                    # Search string
                    if '#EventDate ' in line:
                        EventDate = line.replace("#EventDate ", "").strip()
                        print("Date          :", EventDate)
                        self.labelDate.Text = "Date:  " + EventDate

                    if '#PredictTime ' in line:
                        EventTime = line.replace("#PredictTime ", "").split('; ', 1)[1].split(" ")[0]
                        print("EventTime     :", EventTime, " UT")
                        self.labelTime.Text = "Time:  " + EventTime + " UT"

                    if '#MaxDuration ' in line:
                        MaxDuration = line.replace("#MaxDuration ", "").strip()
                        print("Max. Duration : ", MaxDuration)
                        self.labelMaxDuration.Text = "Max. Duration:  " + MaxDuration

                    if '#AstName ' in line:
                        Ast_Name = line.replace("#AstName ", "").strip()
                        print("AstName       :", Ast_Name)
                        self.labelAst.Text = "Asteroid:  " + Ast_Name

                    if '#StarMag ' in line:
                        Star_Mag = float(line.replace("#StarMag ", ""))
                        self.labelStarMag.Text = "Star Mag:  " + str("%.1f" % round(Star_Mag,1))

                    if '#MagnitudeDrop ' in line:
                        Mag_Drop = float(line.replace("#MagnitudeDrop ", ""))
                        #Ast_Mag = Star_Mag + Mag_Drop
                        I_ratio = pow(2.512, Mag_Drop)
                        Ast_Mag = Star_Mag + 2.5 * log10(I_ratio - 1.0)
                        self.labelAstMag.Text = "Ast Mag:  " + str("%.1f" % round(Ast_Mag,1))

                    if '#ErrorInTime ' in line:
                        ErrorInTime = line.replace("#ErrorInTime ", "").strip()
                        print("Error in Time : ", ErrorInTime)
                        self.labelErrorInTime.Text = "Error in Time:  " + ErrorInTime

                    if '#StarName ' in line:
                        Star_Name = line.replace("#StarName ", "").strip()
                        print("StarName      :", Star_Name)
                        self.labelStar.Text = "Star:  " + Star_Name

                    if '#StarRA ' in line:
                        Star_RA = line.replace("#StarRA ", "")
                        self.labelRA.Text = "RA:  " + Star_RA

                    if '#StarDEC ' in line:
                        Star_DEC = line.replace("#StarDEC ", "").replace("Â", "")
                        self.labelDEC.Text = "DEC:  " + Star_DEC

    def LEDCalibration(self, sender, args):
        with open(LED_Calibration_script) as f:
            exec(f.read())


#BadData
#-------
#Multiple causes? 1) The GPS is reporting some faulty data? Or SharpCap thinks it is faulty? I don't know the
#details for this case. 2) The GPS has partial data only (normally reported as PartialData), but Calibration End Pos
#is also set too high (normally reported as BadCalibrationReduceEndPos if there were a GPS fix), causing the
#reported GPS Time to freeze -- and at a significantly bad value (e.g. year 1995 or 2061). This is a recent issue
#I've discovered; I posted details to the IOTA ilst but not here yet. I can elaborate if desired; this is
#a reproducible scenario -- and will not resolve on its own, even when a GPS fix is achieved.

#PartialData
#-----------
#The GPS sees some satellites, but is still acquiring a fix. Time and position may be (significantly) off.
#(This condition may sometimes be reported as BadData instead.)

#Locked
#------
#The GPS has a fix (precise location and time). However, it may or may not have a full almanac and thus could be
#missing the proper leap second(s) count. So time could actually be off by a small (1-3?) number of seconds --
##but always an exact integral amount. But there is no further indication in GPS Status of when/if the almanac has
#been downloaded (after the initial position fix and `Locked' status). Waiting for Locked, and then waiting another
#12.5 minutes at least (the almanac broadcast repeat cycle), is the only assurance I know of that the leap second
#count (and hence GPS Time) is fully correct. I seem to recall watching the System <-> GPS Time difference change
#by a small integral number of seconds well after Locked was achieved, and have interpreted this as indirect
#evidence that the almanac had then finished downloading (often some several minutes after Locked was achieved).

#BadCalibrationReduceEndPos
#--------------------------
#The GPS has a fix, but Calibration End Pos is set too high; I think the reported GPS time also freezes (but at
#a recent value). (A similar condition -- bad End Pos but no fix -- may sometimes be reported as BadData instead;
#see below.)

#BadTimeStamp
#------------
#The reported GPS time is more than 24 hours different from the system (PC) time, the latter of which is assumed
#to be at least that accurate and thus the GPS time is assumed to be in error. I'm not sure if the GPS has to be
#reporting having a fix for this error to happen; i.e. if no fix, would a greater-than-24-hour time diff be ignored
#and PartialData be reported instead?



    def ShowUTC_Time(self, sender, args):
        global DelayTime, EventDate, EventTime, EventDateTime
        try:
            GPS_status = str(SharpCap.SelectedCamera.LatestStatus.AdditionalStatus["GPS"].Status)
            if (GPS_status == "Locked"):
                q = SharpCap.SelectedCamera.LatestStatus.AdditionalStatus["GPS"].GetGPSTimeNow()
                UTC_Time = datetime(q.Year, q.Month, q.Day, q.Hour, q.Minute, q.Second)
                self.labelUTC.Text = "UTC-GPS: " + GPS_status
            else:
                UTC_Time = datetime.utcnow()
                self.labelUTC.Text = "GPS: " + GPS_status + ", Computer time (UTC):"
        except:
            UTC_Time = datetime.utcnow()
            self.labelUTC.Text = "Computer time (UTC):"

        try:
            DelayTime = int(self.textBoxTimeDelay.Text)
        except:
            DelayTime = 0
        EventDateTime = datetime.strptime(EventDate + " " + EventTime, "%d %B %Y %H:%M:%S")
        delta = round((EventDateTime - UTC_Time + timedelta(seconds= -DelayTime)).total_seconds())

        if (delta == 60):           # 1 minute before start of the capture
            Speak("one minute")
            self.textCountdown.BackColor = Color.Yellow

        if (delta == 10):           # 10 secondes before start of the capture
            Speak("10 seconds")

        if (delta == 0):            # start of capture
            self.textCountdown.BackColor = Color.Red
            if (self.buttonStart.Text == "Stop Countdown"):
                if(not SharpCap.SelectedCamera.Capturing):
                    Speak("Start capture")
                    text = str(2*DelayTime)
                    Speak(text)
                    Speak("seconds")
                    self.StartCapture()
                    self.buttonStart.Text = "Stop Capture"
                    print("Start Capture from Countdown")

        if (delta == -2*DelayTime): # end of capture ... 2x DelayTime
            self.textCountdown.BackColor = Color.White
            if(SharpCap.SelectedCamera.Capturing):
                Speak("Stop capture")
                SharpCap.SelectedCamera.StopCapture()
                print("Stop Capture from Countdown")
                self.InitializeConnection()
                self.WeatherSave(self, sender)
                AfterCapture(self.textBoxImportFile.Text)
            self.buttonStart.Text == "Start Capture"
            self.checkboxAutoCapture.Enabled = True
            self.checkboxAutoCapture.Checked = False

        self.textUTCTime.Text = Roundtime(UTC_Time.ToString().split()[1])
        self.textCountdown.Text = str(timedelta(seconds = abs(delta))).zfill(8)

        if (SharpCap.Mounts.SelectedMount.Connected):
            if (SharpCap.Mounts.SelectedMount.Slewing):
                self.buttonGOTO.Text = "STOP"
                self.buttonGOTO.BackColor = Color.Red
                self.buttonGOTO.ForeColor = Color.White
            else:
                self.buttonGOTO.Text = "GOTO"
                self.buttonGOTO.BackColor = Color.Green
                self.buttonGOTO.ForeColor = Color.White


    def StartCapture(self):
        global EventDateTime
        target_name = self.labelAst.Text.replace("Asteroid:  ", "") + "_" + self.labelStar.Text.replace("Star:  ", "")
        target_name = target_name.replace(" ", "-")
        SharpCap.TargetName = EventDateTime.strftime("%Y%m%d_%H%M%S") + "_" + target_name
        print("Target name: ", SharpCap.TargetName)
        SharpCap.SelectedCamera.PrepareToCapture()
        SharpCap.SelectedCamera.RunCapture()

    def WeatherSave(self, sender, args):
        print("Save information")
        self.InitializeConnection()
        if (self.textBoxImportFile.Text != "Import a file"):
            lines = File.ReadAllLines(self.textBoxImportFile.Text)
            for l_no, line in enumerate(lines):
                # Search string
                if '#Wind:' in line:
                    print("Wind:", self.comboboxWind.Text)
                    if (self.comboboxWind.Text == "unstated"):
                        lines[l_no] = "#Wind: " + ""
                    if (self.comboboxWind.Text == "0: Calm"):
                        lines[l_no] = "#Wind: " + "0"
                    if (self.comboboxWind.Text == "1: Light air"):
                        lines[l_no] = "#Wind: " + "1"
                    if (self.comboboxWind.Text == "2: Light breeze"):
                        lines[l_no] = "#Wind: " + "2"
                    if (self.comboboxWind.Text == "3: Gentle breeze"):
                        lines[l_no] = "#Wind: " + "3"
                    if (self.comboboxWind.Text == "4: Moderate breeze"):
                        lines[l_no] = "#Wind: " + "4"
                    if (self.comboboxWind.Text == "5: Fresh breeze"):
                        lines[l_no] = "#Wind: " + "5"
                    if (self.comboboxWind.Text == "6: Strong breeze"):
                        lines[l_no] = "#Wind: " + "6"
                    if (self.comboboxWind.Text == "7: High wind"):
                        lines[l_no] = "#Wind: " + "7"
                    if (self.comboboxWind.Text == "8: Gale"):
                        lines[l_no] = "#Wind: " + "8"
                    if (self.comboboxWind.Text == "9: Strong gale"):
                        lines[l_no] = "#Wind: " + "9"

                if '#Temperature:' in line:
                    print("Temperature:", self.textBoxTemperature.Text)
                    lines[l_no] = "#Temperature: " + self.textBoxTemperature.Text

                if '#Transparency:' in line:
                    print("Transparency:", self.comboboxTransparency.Text)
                    if (self.comboboxTransparency.Text == "unstated"):
                        lines[l_no] = "#Transparency: " + ""
                    if (self.comboboxTransparency.Text == "Clear"):
                        lines[l_no] = "#Transparency: " + "1"
                    if (self.comboboxTransparency.Text == "Fog"):
                        lines[l_no] = "#Transparency: " + "2"
                    if (self.comboboxTransparency.Text == "Thin cloud"):
                        lines[l_no] = "#Transparency: " + "3"
                    if (self.comboboxTransparency.Text == "Thick cloud"):
                        lines[l_no] = "#Transparency: " + "4"
                    if (self.comboboxTransparency.Text == "Broken opaque cloud"):
                        lines[l_no] = "#Transparency: " + "5"
                    if (self.comboboxTransparency.Text == "Star faint"):
                        lines[l_no] = "#Transparency: " + "6"
                    if (self.comboboxTransparency.Text == "By averted vision"):
                        lines[l_no] = "#Transparency: " + "7"

                if '#Stability:' in line:
                    print("Stability:", self.comboboxStability.Text)
                    if (self.comboboxStability.Text == "unstated"):
                        lines[l_no] = "#Stability: " + ""
                    if (self.comboboxStability.Text == "Steady"):
                        lines[l_no] = "#Stability: " + "1"
                    if (self.comboboxStability.Text == "Slight flickering"):
                        lines[l_no] = "#Stability: " + "2"
                    if (self.comboboxStability.Text == "Strong flickering"):
                        lines[l_no] = "#Stability: " + "3"

                if '#Comments:' in line:
                    print("Comments:", self.textSODISComment.Text.replace("\r","").replace("\n"," "))
                    lines[l_no] = "#Comments: " + self.textSODISComment.Text.replace("\r","").replace("\n"," ")

                if '#Note' in line:
                    print("Note:", self.textNote.Text.replace("\r","").replace("\n"," "))
                    Note_text = "#Note " + self.textNote.Text.replace("\r","").replace("\n"," ") + "\r"
                    Note_text = Note_text + "#UT " + self.textUTCTime.Text.replace(":","-") + "\r"
                    Note_text = Note_text + "#" + self.labelTelStatus.Text.replace(":","") + "\r"
                    Note_text = Note_text + "#" + self.labelFocuser.Text.replace(":","") + "\r"
                    Note_text = Note_text + "#" + self.labelGPS.Text.replace(":","") + "\r"
                    Note_text = Note_text + "#" + self.labelEAGLE.Text.replace(":","")
                    lines[l_no] = Note_text

            File.WriteAllLines(self.textBoxImportFile.Text, lines)

    def TabControl_SelectedIndexChanged(self, sender, args):
        self.InitializeConnection()
        if (self.tabControl.SelectedIndex == 3):
            if (self.buttonStart.Text == "Start Countdown"):
                self.buttonStart.BackColor = Color.Green
                self.buttonStart.ForeColor = Color.White
            if (self.buttonStart.Text == "Stop Countdown"):
                self.buttonStart.BackColor = Color.Red
                self.buttonStart.ForeColor = Color.White

def BeforeClosing(self, sender):
    print("Stop timer before closing")
    self.MyTimerUTC.Stop()


def launch_form():
    form = OccultationMenuForm()
    form.StartPosition = FormStartPosition.CenterScreen
    form.TopMost = True
    form.FormClosing += BeforeClosing
    form.Show()

SharpCap.AddCustomButton(" Occultation  |", Image.FromFile(default_icon), "Occultation Event", launch_form)
